// MIDI specific routines for "MIDI Merger"  **********************************
//
// The MIDI Merger
// merges up to 4 conventional MIDI Inputs into a common MIDI message stream
// to a PC via Ethernet (Multicast or standard UDP) or via USB MIDI Class.
// Vice versa a stream of MIDI messages from a PC is distributed to
// up to 4 conventional MIDI Outputs with channel filter per output.
//
// The design is based on the STM32 Nucleo-L476 module plus a specific PCB.
//
// Author and (c)2017-18 Wolfgang Schemmert. The copyright claim and
// the GPL-3 Licence is restricted to this file and the depending files
// "main.c", "main.h", "usb-otg.c", "ether.c" and "userinterface.c".
// Other depending material is published and copyrighted
// by STMicroelectronics, Atollic AB and EMBITZ.org
//
// Contact: <www.midi-and-more.de>
// A detailled construction and operation manual is provided at this website
//
// Note: at present state of art, only the DEBUG object code is working.
// The RELEASE version compiled with EmBitz 1.11 does not work. Reason unkown.
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32L476xx.h"
#include "globaldefines.h"

// ************************************

//system control
extern uint8_t gOpMode;     //0=Setup, 1=UDP/COM, 2=TCP/COM, 0x1x=MIDI Merge

extern struct PARAM{
uint32_t Vid;
uint32_t Pid;
uint32_t MacHi;
uint32_t MacLo;
uint32_t Gateway;
uint32_t LocalIP;
uint32_t Subnet;
uint32_t UdpDestIP;
uint32_t UdpDestPort;
uint32_t UdpLocalPort;
uint32_t McastIP;
uint32_t McastDestPort;
uint32_t McastLocalPort;
uint32_t LanMode;
uint32_t Midi13Mask;
uint32_t Midi45Mask;
uint32_t EthUsbMask;
uint32_t TcpPort;
uint32_t DisconTime;
uint32_t ComBaud;
uint32_t MidiBaud;
uint32_t Padding;
} gComParam;

extern void servSystem(void);
extern void sendB1(uint8_t);
extern uint16_t pullMIDI1RxBuf(void);
extern uint16_t pullMIDI3RxBuf(void);
extern uint16_t pullMIDI4RxBuf(void);
extern uint16_t pullMIDI5RxBuf(void);
extern void sendB3(uint8_t);
extern void sendB4(uint8_t);
extern void sendB5(uint8_t);
extern uint8_t checkAllUartFree(void);

//extern void sendSetupB(uint8_t); //may be uncommented for debug output
//extern void sendX8(uint8_t);
//extern void sendX16(uint16_t);
//extern void sendX32(uint32_t);


//USB
extern uint8_t  guConfigured;
extern uint8_t guDeviceClass;
extern uint16_t pullUsbRxBuf(void);
extern void enCodeMidi(uint8_t);

//ether.c
extern uint8_t sendEtherByte(uint8_t);
extern uint16_t sendEtherBuffer(uint8_t *, uint16_t);
extern uint16_t pullSpiRxBuf(void);

// ************************************

uint8_t gmMIDI1_MSGBUF[USART_BUFLEN];
uint8_t *gmMIDI1_MSGHEAD, *gmMIDI1_MSGTAIL;
uint8_t gmStatus1, gmChannel1, gmFillStatus1;
uint16_t gmRemain1, gmDataLen1;

uint8_t gmMIDI3_MSGBUF[USART_BUFLEN];
uint8_t *gmMIDI3_MSGHEAD, *gmMIDI3_MSGTAIL;
uint8_t gmStatus3, gmChannel3, gmFillStatus3;
uint16_t gmRemain3, gmDataLen3;

uint8_t gmMIDI4_MSGBUF[USART_BUFLEN];
uint8_t *gmMIDI4_MSGHEAD=0,* gmMIDI4_MSGTAIL=0;
uint8_t gmStatus4, gmChannel4, gmFillStatus4;
uint16_t gmRemain4, gmDataLen4;

uint8_t gmMIDI5_MSGBUF[USART_BUFLEN];
uint8_t *gmMIDI5_MSGHEAD, *gmMIDI5_MSGTAIL;
uint8_t gmStatus5, gmChannel5, gmFillStatus5;
uint16_t gmRemain5, gmDataLen5;

//USB or Ethernet, depending on DIPSW:
uint8_t gmMIDIX_MSGBUF[USART_BUFLEN];
uint8_t *gmMIDIX_MSGHEAD, *gmMIDIX_MSGTAIL;
uint8_t gmStatusX, gmChannelX, gmFillStatusX;
uint16_t gmRemainX, gmDataLenX;

uint8_t gmMIDIS_MSGBUF[USART_BUFLEN];
uint8_t *gmMIDIS_MSGHEAD, *gmMIDIS_MSGTAIL;
uint8_t gmStatusS, gmChannelS, gmFillStatusS;
uint16_t gmRemainS, gmDataLenS;

uint8_t gmDistriChannel,gmBlockF6, gmBlockF8, gmBlockFE;
uint8_t gmSchedulerStage, gmMergeMaster, gmSplitMidi4, gmSplitMidi5;
uint16_t gmMask1, gmMask3, gmMask4, gmMask5, gmMaskEth, gmMaskUsb;

void MidiScheduler(void);
void initMidiBuffers(void);
void realtimeMsgMidi13(uint8_t);
void realtimeMsgMidi45(uint8_t);
void realtimeMsgMidi4(uint8_t);
void realtimeMsgMidi5(uint8_t);
void msgMidiX(void);
void msgMidi1(void);
void msgMidi3(void);
void msgMidi4(void);
void msgMidi5(void);
void distributeMidiOut(void);
void sendSplitMsg(uint8_t);
void msgMidiS(void);
void sendAllMidi1345X(uint8_t);
void distributeMidi1345X(uint8_t);
void distributeMidi13(uint8_t);
void distributeMidi45(uint8_t);
void legacyMidiScheduler(void);
void legacyMsgMidi1(void);
void legacyMsgMidi3(void);
void legacyMsgMidiX(void);

void MidiScheduler(void) // *************************************************
//gOpMode=0x11,0x12,0x13: MIDI Merge with normal UDP
//gOpMode=0x21,0x22,0x23: MIDI Merge with UDP Multicast
//gmMergeMaster=1: only left DIPSW is ON: Ethernet/UDP is Merge Master
//gmMergeMaster=2: only right DIPSW is ON: USB is Merge Master
//gMergeMaster=3: both DIPSW are ON;UDP & USB are split Merge Masters
//SplitMerge means:
//MIDI1 and MIDI3 are merged(sent) to Ethernet,MIDI4 and MIDI5 are merged to USB
//UDP is distributed to MIDI1 and MIDI3, USB is distributed to MIDI4 and MIDI5
// **************************************************************************
{
uint8_t byteCount, recvbyte;
uint16_t recvword;

    gmSchedulerStage++;   //provisorisch
    if (gmSchedulerStage > 4) gmSchedulerStage = 0; //Robin Round scheme

    if (gmSchedulerStage == 0) {
//this handles the 5th merger input (USB when Ethernet=master and vice versa)
        if (gmMergeMaster == 3)  return;    //no 5th MIDI IN in this case

//left DIPSW is ON (gmMergeMaster==1): Ethernet is Master, USB is 5th merged
//right DIPSW is ON(gmMergeMaster==2): USB is Master, Ethernet is 5th merged
        byteCount = 0;
        while (byteCount < 3)  {      //process max.3bytes per task robin round

            if (gmMergeMaster == 1)  {      //Ethernet is MergeMaster,USB is 5th
                if ((recvword = pullUsbRxBuf()) == 0) break;
            }
            else  {                         //USB is MergeMaster,Ethernet is 5th
            if ((recvword = pullSpiRxBuf()) == 0) break;
            }
            recvbyte = (uint8_t)(recvword & 0x00FF);

            if (recvbyte < 0x80) {
                if (gmRemainX == 4)  {          //only possible with long SysEX
                    msgMidiX();                 //merge is not safe anymore !!
                    gmRemainX = USART_BUFLEN;
                    gmDataLenX = USART_BUFLEN;
                }
                if (gmRemainX == 0)  break;                 //ignore wild bytes

                 if (gmFillStatusX) {                       //is in RunningState
                    *gmMIDIX_MSGHEAD++ = gmStatusX | gmChannelX;
                    gmFillStatusX = 0;
                }
                *gmMIDIX_MSGHEAD++ = recvbyte;
                gmRemainX--;
                if (gmRemainX == 0)  {
                    msgMidiX();
                    gmRemainX = gmDataLenX;                 //for Running State
                    gmFillStatusX = 0xFF;
                    break;
                }
            }
            else {
                gmFillStatusX = 0;
                if ((recvbyte == 0xF7) && (gmStatusX == 0xF0)) { //end of Sysex
                    *gmMIDIX_MSGHEAD++ = recvbyte;
                    msgMidiX();
                    gmRemainX = 0;
                    break;
                }
                else if (recvbyte >= 0xF4)  realtimeMsgMidi45(recvbyte);
                                                            //single byte msg
                                //even forward undefined messages immediately
                else {
                    gmMIDIX_MSGHEAD = gmMIDIX_MSGBUF;   //ignore uncompl. msgs
                    *gmMIDIX_MSGHEAD++ = recvbyte;
                    gmRemainX = 0;
                    gmStatusX = recvbyte & 0xF0;
                    gmChannelX = recvbyte & 0x0F;
                    if (recvbyte == 0xF0) gmRemainX = USART_BUFLEN;
                    if ((recvbyte == 0xF1) || (recvbyte == 0xF3)) gmRemainX = 1;
                    if ((gmStatusX == 0xC0)||(gmStatusX == 0xD0)) gmRemainX = 1;
                    if (gmRemainX == 0) gmRemainX = 2;      //=3byte MIDI Msgs
                    gmDataLenX = gmRemainX;
                }
            }
            byteCount++;
        }
//distribute bytes from Merge Master:
        if (checkAllUartFree()) distributeMidiOut();
        return;
    }

//and now check the classic MIDI inputs:
    if (gmSchedulerStage == 1) {
        byteCount = 0;
        while (byteCount < 3)  {    //process max.3bytes per task robin round

            if ((recvword = pullMIDI1RxBuf()) == 0) break;
            recvbyte = (uint8_t)(recvword & 0x00FF);

            if (recvbyte < 0x80) {
                if (gmRemain1 == 4)  {          //only possible with long SysEX
                    msgMidi1();                 //merge is not safe anymore !!
                    gmRemain1 = USART_BUFLEN;
                    gmDataLen1 = USART_BUFLEN;
                }
                if (gmRemain1 == 0)  break;                 //ignore wild bytes

                if (gmFillStatus1) {                        //is in RunningState
                    *gmMIDI1_MSGHEAD++ = gmStatus1 | gmChannel1;
                    gmFillStatus1 = 0;
                }
                *gmMIDI1_MSGHEAD++ = recvbyte;
                gmRemain1--;
                if (gmRemain1 == 0)  {
                    msgMidi1();
                    gmRemain1 = gmDataLen1;                 //for Running State
                    gmFillStatus1 = 0xFF;
                    break;
                }
            }
            else {
                gmFillStatus1 = 0;
                if ((recvbyte == 0xF7) && (gmStatus1 == 0xF0)) { //end of Sysex
                    *gmMIDI1_MSGHEAD++ = recvbyte;
                    msgMidi1();
                    gmRemain1 = 0;
                    gmDataLen1 = 0;
                    break;
                }
                else if (recvbyte >= 0xF4) realtimeMsgMidi13(recvbyte);
                                                            //single byte msg
                                //even forward undefined messages immediately
                else {
                    gmMIDI1_MSGHEAD = gmMIDI1_MSGBUF; //ignore uncomplete msgs
                    *gmMIDI1_MSGHEAD++ = recvbyte;
                    gmRemain1 = 0;
                    gmStatus1 = recvbyte & 0xF0;
                    gmChannel1 = recvbyte & 0x0F;
                    if (recvbyte == 0xF0) gmRemain1 = USART_BUFLEN;
                    if ((recvbyte == 0xF1) || (recvbyte == 0xF3)) gmRemain1 = 1;
                    if ((gmStatus1 == 0xC0)||(gmStatus1 == 0xD0)) gmRemain1 = 1;
                    if (gmRemain1 == 0) gmRemain1 = 2;      //=3byte MIDI Msgs
                    gmDataLen1 = gmRemain1;
                }
            }
            byteCount++;
        }
//distribute bytes from Merge Master:
        if (checkAllUartFree()) distributeMidiOut();
        return;
    }

    if (gmSchedulerStage == 2) {
       byteCount = 0;
        while (byteCount < 3)  {    //process max.3bytes per task robin round

            if ((recvword = pullMIDI3RxBuf()) == 0) break;
            recvbyte = (uint8_t)(recvword & 0x00FF);

            if (recvbyte < 0x80) {
                if (gmRemain3 == 4)  {          //only possible with long SysEX
                    msgMidi3();                 //merge is not safe anymore !!
                    gmRemain3 = USART_BUFLEN;
                    gmDataLen3 = USART_BUFLEN;
                }
                if (gmRemain3 == 0)  break;                 //ignore wild bytes

                if (gmFillStatus3) {                        //is in RunningState
                    *gmMIDI3_MSGHEAD++ = gmStatus3 | gmChannel3;
                    gmFillStatus3 = 0;
                }
                *gmMIDI3_MSGHEAD++ = recvbyte;

                gmRemain3--;
                if (gmRemain3 == 0)  {
                    msgMidi3();
                    gmRemain3 = gmDataLen3;                 //for Running State
                    gmFillStatus3 = 0xFF;
                    break;
                }
            }
            else {                                          //Status Byte
                gmFillStatus3 = 0;
                if ((recvbyte == 0xF7) && (gmStatus3 == 0xF0)) { //end of Sysex
                    *gmMIDI3_MSGHEAD++ = recvbyte;
                    msgMidi3();
                    gmRemain3 = 0;
                    gmDataLen3 = 0;
                    break;
                }
                else if (recvbyte >= 0xF4) realtimeMsgMidi13(recvbyte);
                                                            //single byte msg
                                //even forward undefined messages immediately
                else {
                    gmMIDI3_MSGHEAD = gmMIDI3_MSGBUF; //ignore uncomplete msgs
                    *gmMIDI3_MSGHEAD++ = recvbyte;
                    gmRemain3 = 0;
                    gmStatus3 = recvbyte & 0xF0;
                    gmChannel3 = recvbyte & 0x0F;
                    if (recvbyte == 0xF0) gmRemain3 = USART_BUFLEN;
                    if ((recvbyte == 0xF1) || (recvbyte == 0xF3)) gmRemain3 = 1;
                    if ((gmStatus3 == 0xC0)||(gmStatus3 == 0xD0)) gmRemain3 = 1;
                    if (gmRemain3 == 0) gmRemain3 = 2;      //=3byte MIDI Msgs
                    gmDataLen3 = gmRemain3;
                }
            }
            byteCount++;
        }
//distribute bytes from Merge Master:
        if (checkAllUartFree()) distributeMidiOut();
        return;
    }

    if (gmSchedulerStage == 3) {
        byteCount = 0;
        while (byteCount < 3)  {    //process max.3bytes per task robin round

            if ((recvword = pullMIDI4RxBuf()) == 0) break;
            recvbyte = (uint8_t)(recvword & 0x00FF);

            if (recvbyte < 0x80) {
                if (gmRemain4 == 4)  {          //only possible with long SysEX
                    msgMidi4();                 //merge is not safe anymore !!
                    gmRemain4 = USART_BUFLEN;
                    gmDataLen4 = USART_BUFLEN;
                }
                if (gmRemain4 == 0)  break;                 //ignore wild bytes

                if (gmFillStatus4) {                        //is in RunningState
                    *gmMIDI4_MSGHEAD++ = gmStatus4 | gmChannel4;
                    gmFillStatus4 = 0;
                }
                *gmMIDI4_MSGHEAD++ = recvbyte;
                gmRemain4--;
                if (gmRemain4 == 0)  {
                    msgMidi4();
                    gmRemain4 = gmDataLen4;                 //for Running State
                    gmFillStatus4 = 0xFF;
                    break;
                }
            }
            else {
                gmFillStatus4 = 0;
                if ((recvbyte == 0xF7) && (gmStatus4 == 0xF0)) { //end of Sysex
                    *gmMIDI4_MSGHEAD++ = recvbyte;
                    msgMidi4();
                    gmRemain4 = 0;
                    gmDataLen4 = 0;
                    break;
                }
                else if (recvbyte >= 0xF4) {                //single byte msg
                                //even forward undefined messages immediately
                    realtimeMsgMidi45(recvbyte);
                    if ((gmSplitMidi4) && (gmMergeMaster != 3)) {
                        sendB1(recvbyte);
                        sendB3(recvbyte);
                        sendB5(recvbyte);
                    }
                }
                else {
                    gmMIDI4_MSGHEAD = gmMIDI4_MSGBUF; //ignore uncomplete msgs
                    *gmMIDI4_MSGHEAD++ = recvbyte;
                    gmRemain4 = 0;
                    gmStatus4 = recvbyte & 0xF0;
                    gmChannel4 = recvbyte & 0x0F;
                    if (recvbyte == 0xF0) gmRemain4 = USART_BUFLEN;
                    if ((recvbyte == 0xF1) || (recvbyte == 0xF3)) gmRemain4 = 1;
                    if ((gmStatus4 == 0xC0)||(gmStatus4 == 0xD0)) gmRemain4 = 1;
                    if (gmRemain4 == 0) gmRemain4 = 2;      //=3byte MIDI Msgs
                    gmDataLen4 = gmRemain4;
                }
            }
            byteCount++;
        }
//distribute bytes from Merge Master:
        if (checkAllUartFree()) distributeMidiOut();
        return;
    }

    if (gmSchedulerStage == 4) {
       byteCount = 0;
        while (byteCount < 3)  {    //process max.3bytes per task robin round

            if ((recvword = pullMIDI5RxBuf()) == 0) break;
            recvbyte = (uint8_t)(recvword & 0x00FF);

            if (recvbyte < 0x80) {
                if (gmRemain5 == 4)  {          //only possible with long SysEX
                    msgMidi5();                 //merge is not safe anymore !!
                    gmRemain5 = USART_BUFLEN;
                    gmDataLen5 = USART_BUFLEN;
                }
                if (gmRemain5 == 0)  break;                 //ignore wild bytes

                if (gmFillStatus5) {                        //is in RunningState
                    *gmMIDI5_MSGHEAD++ = gmStatus5 | gmChannel5;
                    gmFillStatus5 = 0;
                }
                *gmMIDI5_MSGHEAD++ = recvbyte;
                gmRemain5--;
                if (gmRemain5 == 0)  {
                    msgMidi5();
                    gmRemain5 = gmDataLen5;                 //for Running State
                    gmFillStatus5 = 0xFF;
                    break;
                }
            }
            else {                                          //new Status Byte
                gmFillStatus5 = 0;
                if ((recvbyte == 0xF7) && (gmStatus5 == 0xF0)) { //end of Sysex
                    *gmMIDI5_MSGHEAD++ = recvbyte;
                    msgMidi5();
                    gmRemain5 = 0;
                    gmDataLen5 = 0;
                    break;
                }
                else if (recvbyte >= 0xF4) {                //single byte msg
                                //even forward undefined messages immediately
                    realtimeMsgMidi45(recvbyte);
                    if ((gmSplitMidi5) && (gmMergeMaster != 3))  {
                        sendB1(recvbyte);
                        sendB3(recvbyte);
                        sendB4(recvbyte);
                    }
                }
                else {
                    gmMIDI5_MSGHEAD = gmMIDI5_MSGBUF; //ignore uncomplete msgs
                    *gmMIDI5_MSGHEAD++ = recvbyte;
                    gmRemain5 = 0;

                    gmStatus5 = recvbyte & 0xF0;
                    gmChannel5 = recvbyte & 0x0F;
                    if (recvbyte == 0xF0) gmRemain5 = USART_BUFLEN;
                    if ((recvbyte == 0xF1) || (recvbyte == 0xF3)) gmRemain5 = 1;
                    if ((gmStatus5 == 0xC0)||(gmStatus5 == 0xD0)) gmRemain5 = 1;
                    if (gmRemain5 == 0) gmRemain5 = 2;      //=3byte MIDI Msgs
                    gmDataLen5 = gmRemain5;
                }
            }
            byteCount++;
        }
//distribute bytes from Merge Master:
        if (checkAllUartFree()) distributeMidiOut();
        return;
    }
}


void initMidiBuffers(void)  // **********************************************
// **************************************************************************
{
    gmMIDI1_MSGHEAD = gmMIDI1_MSGBUF;
    gmMIDI3_MSGHEAD = gmMIDI3_MSGBUF;
    gmMIDI4_MSGHEAD = gmMIDI4_MSGBUF;
    gmMIDI5_MSGHEAD = gmMIDI5_MSGBUF;
    gmMIDIX_MSGHEAD = gmMIDIX_MSGBUF;
    gmMIDIS_MSGHEAD = gmMIDIS_MSGBUF;

    gmRemain1 = 0;
    gmRemain3 = 0;
    gmRemain4 = 0;
    gmRemain5 = 0;
    gmRemainX = 0;
    gmRemainS = 0;

    gmMask1 = (uint16_t)(gComParam.Midi13Mask >> 16);
    gmMask3 = (uint16_t)(gComParam.Midi13Mask & 0x0000FFFF);
    gmMask4 = (uint16_t)(gComParam.Midi45Mask >> 16);
    gmMask5 = (uint16_t)(gComParam.Midi45Mask & 0x0000FFFF);
    gmMaskEth = (uint16_t)(gComParam.EthUsbMask >> 16);
    gmMaskUsb = (uint16_t)(gComParam.EthUsbMask & 0x0000FFFF);
    gmSchedulerStage = 0;

}

void realtimeMsgMidi45(uint8_t data)  // *************************************
//send special byte passing along packet buffering
// **************************************************************************
{
    if (gmMergeMaster == 1) {                   //left DIPSW is ON
//Ethernet is Merge Collector (receiver of merged msgs) for MIDI1 and MIDI3
        sendEtherByte(data);
    }
    else  {                                     //right DIPSW or both are ON
//USB is Merge Collector (receiver of merged msgs) for MIDI1 and MIDI3
        enCodeMidi(data);
    }
}


void realtimeMsgMidi13(uint8_t data)  // *************************************
//send special byte from MIDI1 or MIDI3 IN passing along packet buffering
// **************************************************************************
{
    if (gmMergeMaster == 2 )  enCodeMidi(data); //right DIPSW is ON
    else  {                                     //left DIPSW or both are ON
        sendEtherByte(data);
    }
}


void msgMidiX(void)  // *****************************************************
// **************************************************************************
{
uint16_t toSend, offset;

    if (gmMergeMaster == 1)  {                     //left DIPSW is ON
//Ethernet is Merge Master
        gmMIDIX_MSGTAIL = gmMIDIX_MSGBUF;
        while ((gmMIDIX_MSGHEAD - gmMIDIX_MSGTAIL) >= SPI_BUFLEN - 4 )  {
            sendEtherBuffer(gmMIDIX_MSGTAIL, SPI_BUFLEN - 4);
            gmMIDIX_MSGTAIL += SPI_BUFLEN - 4;
        }
        if ((gmMIDIX_MSGHEAD - gmMIDIX_MSGTAIL) != 0)  {
        sendEtherBuffer(gmMIDIX_MSGTAIL,
                                (uint16_t)(gmMIDIX_MSGHEAD - gmMIDIX_MSGTAIL));
        }
    }
    else  {                                         //right DIPSW is ON
//USB is Merge Master
        toSend = (uint16_t)(gmMIDIX_MSGHEAD - gmMIDIX_MSGBUF);
        offset = 0;
        while (toSend)  {
            enCodeMidi(gmMIDIX_MSGBUF[offset]);
            toSend--;
            offset++;
        }
    }                                               //both DIPSW are ON
    gmMIDIX_MSGHEAD = gmMIDIX_MSGBUF;               //reset Pointer
}


void msgMidi1(void)  // *****************************************************
// **************************************************************************
{
uint16_t toSend, offset;

     if (gmMergeMaster == 2) {                  //right DIPSW is ON
//USB is Merge Collector (receiver of merged msgs) for MIDI1
        toSend = (uint16_t)(gmMIDI1_MSGHEAD - gmMIDI1_MSGBUF);
        offset = 0;
        while (toSend)  {
            enCodeMidi(gmMIDI1_MSGBUF[offset]);
            toSend--;
            offset++;
        }
    }
    else  {                                     //left DIPSW or both are ON
//Ethernet is Merge Collector (receiver of merged msgs) for all MIDI IN
        gmMIDI1_MSGTAIL = gmMIDI1_MSGBUF;
        while ((gmMIDI1_MSGHEAD - gmMIDI1_MSGTAIL) >= SPI_BUFLEN - 4 )  {
            sendEtherBuffer(gmMIDI1_MSGTAIL, SPI_BUFLEN - 4);
            gmMIDI1_MSGTAIL += SPI_BUFLEN - 4;
        }
        if ((gmMIDI1_MSGHEAD - gmMIDI1_MSGTAIL) != 0)  {
        sendEtherBuffer(gmMIDI1_MSGTAIL,
                                (uint16_t)(gmMIDI1_MSGHEAD - gmMIDI1_MSGTAIL));
        }
    }
    gmMIDI1_MSGHEAD = gmMIDI1_MSGBUF;           //reset Pointer
}


void msgMidi3(void)  // *****************************************************
// **************************************************************************
{
uint16_t toSend, offset;

    if (gmMergeMaster == 2) {                   //right DIPSW is ON
//USB is Merge Collector (receiver of merged msgs) for MIDI3
        toSend = (uint16_t)(gmMIDI3_MSGHEAD - gmMIDI3_MSGBUF);
        offset = 0;
        while (toSend)  {
            enCodeMidi(gmMIDI3_MSGBUF[offset]);
            toSend--;
            offset++;
        }
    }
    else  {                                     //left DIPSW or both are ON
//Ethernet is Merge Collector (receiver of merged msgs) for all MIDI IN
        gmMIDI3_MSGTAIL = gmMIDI3_MSGBUF;
        while (((uint32_t)(gmMIDI3_MSGHEAD-gmMIDI3_MSGTAIL)) >= (SPI_BUFLEN-4)){
            sendEtherBuffer(gmMIDI3_MSGTAIL, SPI_BUFLEN - 4);
            gmMIDI3_MSGTAIL += SPI_BUFLEN - 4;
        }
        if (gmMIDI3_MSGHEAD != gmMIDI3_MSGTAIL)  {
        sendEtherBuffer(gmMIDI3_MSGTAIL,
                                (uint16_t)(gmMIDI3_MSGHEAD - gmMIDI3_MSGTAIL));
        }
    }
    gmMIDI3_MSGHEAD = gmMIDI3_MSGBUF;           //reset Pointer
}


void msgMidi4(void)  // *****************************************************
// **************************************************************************
{
uint16_t toSend, sending, offset, filter;
uint8_t splitChannel, data;

    toSend = (uint16_t)(gmMIDI4_MSGHEAD - gmMIDI4_MSGBUF);

    if (gmMergeMaster == 1) {                   //left DIPSW is ON
//Ethernet is Merge Collector (receiver of merged msgs) for all MIDI4
        gmMIDI4_MSGTAIL = gmMIDI4_MSGBUF;
        while ((gmMIDI4_MSGHEAD - gmMIDI4_MSGTAIL) >= SPI_BUFLEN - 4 )  {
            sendEtherBuffer(gmMIDI4_MSGTAIL, SPI_BUFLEN - 4);
            gmMIDI4_MSGTAIL += SPI_BUFLEN - 4;
        }
        if ((gmMIDI4_MSGHEAD - gmMIDI4_MSGTAIL) != 0)  {
        sendEtherBuffer(gmMIDI4_MSGTAIL,
                                (uint16_t)(gmMIDI4_MSGHEAD - gmMIDI4_MSGTAIL));
        }
    }
    else  {                                     //right DIPSW or both are ON
//USB is Merge Collector (receiver of merged msgs) for all MIDI IN
        sending = toSend;
        offset = 0;
        while (sending)  {
            enCodeMidi(gmMIDI4_MSGBUF[offset]);
            sending--;
            offset++;
        }
    }

    if ((gmSplitMidi4) && (gmMergeMaster != 3))  {
        offset = 0;
        while (toSend)  {
            data = gmMIDI4_MSGBUF[offset];
            toSend--;
            offset++;
            if (data >= 0xF0)  {
                splitChannel = 0xFF;
                sendB1(data);
                sendB3(data);
                sendB5(data);
            }
            else  if (data >= 0x80)  {
                splitChannel = data & 0x0F;
                filter = 1<<(15 - splitChannel);
                if (gmMask1 & filter) sendB1(data);
                if (gmMask3 & filter) sendB3(data);
                if (gmMask5 & filter) sendB5(data);
            }
//handle bytes < 0x80:
            else if (splitChannel == 0xFF) {  //SysEx body
                sendB1(data);
                sendB3(data);
                sendB5(data);
            }
            else  {
//                filter = 1<<(15 - splitChannel);
                if (gmMask1 & filter) sendB1(data);
                if (gmMask3 & filter) sendB3(data);
                if (gmMask5 & filter) sendB5(data);
            }
        }
    }
    gmMIDI4_MSGHEAD = gmMIDI4_MSGBUF;           //reset Pointer
}


void msgMidi5(void)  // *****************************************************
// **************************************************************************
{
uint16_t toSend, sending, offset, filter;
uint8_t splitChannel, data;

    toSend = (uint16_t)(gmMIDI5_MSGHEAD - gmMIDI5_MSGBUF);

    if (gmMergeMaster == 1) {                   //left DIPSW is ON
//Ethernet is Merge Collector (receiver of merged msgs) for all MIDI5
        gmMIDI5_MSGTAIL = gmMIDI5_MSGBUF;
        while ((gmMIDI5_MSGHEAD - gmMIDI5_MSGTAIL) >= SPI_BUFLEN - 4 )  {
            sendEtherBuffer(gmMIDI5_MSGTAIL, SPI_BUFLEN - 4);
            gmMIDI5_MSGTAIL += SPI_BUFLEN - 4;
        }
        if ((gmMIDI5_MSGHEAD - gmMIDI5_MSGTAIL) != 0)  {
        sendEtherBuffer(gmMIDI5_MSGTAIL,
                                (uint16_t)(gmMIDI5_MSGHEAD - gmMIDI5_MSGTAIL));
        }
    }
    else  {                                     //right DIPSW or both are ON
//USB is Merge Collector (receiver of merged msgs) for all MIDI IN
        sending = toSend;
        offset = 0;
        while (sending)  {
            enCodeMidi(gmMIDI5_MSGBUF[offset]);
            sending--;
            offset++;
        }
    }

    if ((gmSplitMidi5) && (gmMergeMaster != 3))  {
        offset = 0;
        while (toSend)  {
            data = gmMIDI5_MSGBUF[offset];
            toSend--;
            offset++;
            if (data >= 0xF0)  {
                splitChannel = 0xFF;
                sendB1(data);
                sendB3(data);
                sendB4(data);
            }
            else  if (data >= 0x80)  {
                splitChannel = data & 0x0F;
                filter = 1<<(15 - splitChannel);
                if (gmMask1 & filter) sendB1(data);
                if (gmMask3 & filter) sendB3(data);
                if (gmMask4 & filter) sendB4(data);
            }
//handle bytes < 0x80:
            else if (splitChannel == 0xFF) {  //SysEx body
                sendB1(data);
                sendB3(data);
                sendB4(data);
            }
            else  {
 //               filter = 1<<(15 - splitChannel);
                if (gmMask1 & filter) sendB1(data);
                if (gmMask3 & filter) sendB3(data);
                if (gmMask4 & filter) sendB4(data);
            }
        }
    }
    gmMIDI5_MSGHEAD = gmMIDI5_MSGBUF;           //reset Pointer
}


void distributeMidiOut(void)  // *********************************************
// **************************************************************************
{
uint8_t distribyte;
uint16_t distriword;

    if (gmMergeMaster == 3)  {                            //both DIPSW are ON
//Split Merge Mode:
//Ethernet messages are retransmitted to MIDI1 and MIDI3
//USB messages are retransmitted to MIDI4 and MIDI5
        distriword = pullSpiRxBuf();
        if (distriword)  {
            distribyte = (uint8_t)(distriword & 0x00FF);
            if (distribyte >= 0xF0)  {
                gmDistriChannel = 0xFF;
                sendB1(distribyte);
                sendB3(distribyte);
            }
            else  if (distribyte >= 0x80)  {
                gmDistriChannel = distribyte & 0x0F;
                distributeMidi13(distribyte);
            }
//handle bytes < 0x80:
            else if (gmDistriChannel == 0xFF) {
                sendB1(distribyte);
                sendB3(distribyte);
            }
            else distributeMidi13(distribyte);

        }
        distriword = 0;
        if (guConfigured) distriword = pullUsbRxBuf();
        if (distriword)  {
            distribyte = (uint8_t)(distriword & 0x00FF);
            if (distribyte >= 0xF0)  {
                gmDistriChannel = 0xFF;
                sendB4(distribyte);
                sendB5(distribyte);

            }
            else  if (distribyte >= 0x80)  {
                gmDistriChannel = distribyte & 0x0F;
                distributeMidi45(distribyte);
            }
//handle bytes < 0x80:
            else if (gmDistriChannel == 0xFF)  {
                sendB4(distribyte);
                sendB5(distribyte);
            }
            else distributeMidi45(distribyte);
        }
        return;
    }

    if (gmMergeMaster == 1) {           //left DIPSW is ON
//Ethernet messages are retransmitted to all MIDI OUT
            distriword = pullSpiRxBuf();
    }
    else if (gmMergeMaster == 2) {      //right DIPSW is ON
//USB messages are retransmitted to all MIDI OUT
            distriword = 0;
            if (guConfigured) distriword = pullUsbRxBuf();
    }
    if (distriword) {
        distribyte = (uint8_t)(distriword & 0x00FF);

        if ((gmSplitMidi4 == 0) && (gmSplitMidi5 == 0))  {
            if (distribyte >= 0xF0)  {
                gmDistriChannel = 0xFF;
                sendAllMidi1345X(distribyte);
            }
            else  if (distribyte >= 0x80)  {
                gmDistriChannel = distribyte & 0x0F;
                distributeMidi1345X(distribyte);
            }
//handle bytes < 0x80:
            else if (gmDistriChannel == 0xFF) sendAllMidi1345X(distribyte);
            else distributeMidi1345X(distribyte);
        }
        else sendSplitMsg(distribyte);
    }
}


// special merge features when setup commands J or K are active:

void sendSplitMsg(uint8_t byte)   // ****************************************
// if MIDI4 IN or MIDI5 IN is merged
// the distributed message from MergeMaster must be buffered
// else no buffering is necessary for distribution
// **************************************************************************
{
    if (byte < 0x80) {
        if (gmRemainS == 4)  {          //only possible with long SysEX
            msgMidiS();                 //merge is not safe anymore !!
            gmRemainS = USART_BUFLEN;
            gmDataLenS = USART_BUFLEN;
        }
        if (gmRemainS == 0) return;                         //ignore wild bytes

        if (gmFillStatusS) {
            *gmMIDIS_MSGHEAD++ = gmStatusS | gmChannelS;
            gmFillStatusS = 0;
        }
        *gmMIDIS_MSGHEAD++ = byte;
        gmRemainS--;
        if (gmRemainS == 0)  {
            msgMidiS();
            gmRemainS = gmDataLenS;                         //for Running State
            gmFillStatusS = 0xFF;
            return;
        }
    }
    else {
        gmFillStatusS = 0;
        if ((byte == 0xF7) && (gmStatusS == 0xF0)) {        //terminate Sysex
            *gmMIDIS_MSGHEAD++ = byte;
            msgMidiS();
            gmRemainS = 0;
            return;
        }
        else if (byte >= 0xF4)  sendAllMidi1345X(byte);     //single byte msg
                                //even forward undefined messages immediately
        else {                                  //new status byte except Sysex
            gmMIDIS_MSGHEAD = gmMIDIS_MSGBUF;   //ignore uncomplete msgs
            *gmMIDIS_MSGHEAD++ = byte;
            gmRemainS = 0;
            gmStatusS = byte & 0xF0;
            gmChannelS = byte & 0x0F;
            if (byte == 0xF0) gmRemainS = USART_BUFLEN;
            if ((byte == 0xF1) || (byte == 0xF3)) gmRemainS = 1;
            if ((gmStatusS == 0xC0)||(gmStatusS == 0xD0)) gmRemainS = 1;
            if (gmRemainS == 0) gmRemainS = 2;              //=3byte MIDI Msgs
            gmDataLenS = gmRemainS;
        }
    }
}


void msgMidiS(void)  // *****************************************************
// special buffered transmission to avoid conflict with merged MIDI4 or MIDI5
// **************************************************************************
{
uint16_t toSend, offset;
uint8_t distribyte;

    toSend = (uint16_t)(gmMIDIS_MSGHEAD - gmMIDIS_MSGBUF);
    offset = 0;
    while (toSend) {
        distribyte = gmMIDIS_MSGBUF[offset];
        if (distribyte >= 0xF0)  {
            gmDistriChannel = 0xFF;
            sendAllMidi1345X(distribyte);
        }
        else  if (distribyte >= 0x80)  {
            gmDistriChannel = distribyte & 0x0F;
            distributeMidi1345X(distribyte);
        }
//handle bytes < 0x80:
        else if (gmDistriChannel == 0xFF) sendAllMidi1345X(distribyte);
        else distributeMidi1345X(distribyte);
        toSend--;
        offset++;
    }
    gmMIDIS_MSGHEAD = gmMIDIS_MSGBUF;           //reset Pointer
}

void sendAllMidi1345X(uint8_t data)  // *************************************
// **************************************************************************
{
    sendB1(data);
    sendB3(data);
    sendB4(data);
    sendB5(data);
    if (gmMergeMaster == 1) enCodeMidi(data);
    else sendEtherByte(data);
}


void distributeMidi1345X(uint8_t data)  // **********************************
// **************************************************************************
{
uint16_t filter;

        filter = 1<<(15 - gmDistriChannel);
        if (gmMask1 & filter) sendB1(data);
        if (gmMask3 & filter) sendB3(data);
        if (gmMask4 & filter) sendB4(data);
        if (gmMask5 & filter) sendB5(data);
        if ((gmMergeMaster == 1) && (gmMaskUsb & filter)) enCodeMidi(data);
        if ((gmMergeMaster == 2) && (gmMaskEth & filter)) sendEtherByte(data);
}


void distributeMidi13(uint8_t data)  // *************************************
// **************************************************************************
{
uint16_t filter;

        filter = 1<<(15 - gmDistriChannel);
        if (gmMask1 & filter) sendB1(data);
        if (gmMask3 & filter) sendB3(data);
}


void distributeMidi45(uint8_t data)  // *************************************
// **************************************************************************
{
uint16_t filter;

        filter = 1<<(15 - gmDistriChannel);
        if (gmMask4 & filter) sendB4(data);
        if (gmMask5 & filter) sendB5(data);

}


void legacyMidiScheduler(void) // ********************************************
//MIDI1 and MIDI3 are merged(sent) to MIDI5
//MIDI5 is distributed to MIDI1 and MIDI3
//new as of Revnum 4: USB or Ethernet can be merged to MIDI5 too
// **************************************************************************
{
uint8_t byteCount, recvbyte;
uint16_t recvword;

    gmSchedulerStage++;
    if (gmSchedulerStage > 2) gmSchedulerStage = 0; //Robin Round scheme

    if (gmSchedulerStage == 0) {

        byteCount = 0;
        while (byteCount < 3)  {    //process max.3bytes per task robin round

            if ((recvword = pullMIDI1RxBuf()) == 0) break;
            recvbyte = (uint8_t)(recvword & 0x00FF);

            if (recvbyte < 0x80) {
                if (gmRemain1 == 4)  {          //only possible with long SysEX
                    msgMidi1();                 //merge is not safe anymore !!
                    gmRemain1 = USART_BUFLEN;
                    gmDataLen1 = USART_BUFLEN;
                }
                if (gmRemain1 == 0)  break;                 //ignore wild bytes

                if (gmFillStatus1) {                        //is in RunningState
                    *gmMIDI1_MSGHEAD++ = gmStatus1 | gmChannel1;
                    gmFillStatus1 = 0;
                }
                *gmMIDI1_MSGHEAD++ = recvbyte;
                gmRemain1--;
                if (gmRemain1 == 0)  {
                    legacyMsgMidi1();
                    gmRemain1 = gmDataLen1;                 //for Running State
                    gmFillStatus1 = 0xFF;
                    break;
                }
            }
            else {
                gmFillStatus1 = 0;              //start collecting new message
                if ((recvbyte == 0xF7) && (gmStatus1 == 0xF0)) { //end of Sysex
                    *gmMIDI1_MSGHEAD++ = recvbyte;
                    legacyMsgMidi1();
                    gmRemain1 = 0;
                    gmDataLen1 = 0;
                    break;
                }
                else if (recvbyte >= 0xF4) sendB5(recvbyte); //single byte msg
                            //MIDI5 is exclusive msg collector for all MIDI IN
                            //even forward undefined messages immediately
                else {
                    gmMIDI1_MSGHEAD = gmMIDI1_MSGBUF; //ignore uncomplete msgs
                    *gmMIDI1_MSGHEAD++ = recvbyte;
                    gmRemain1 = 0;
                    gmStatus1 = recvbyte & 0xF0;
                    gmChannel1 = recvbyte & 0x0F;
                    if (recvbyte == 0xF0) gmRemain1 = USART_BUFLEN; //max length
                    if ((recvbyte == 0xF1) || (recvbyte == 0xF3)) gmRemain1 = 1;
                    if ((gmStatus1 == 0xC0)||(gmStatus1 == 0xD0)) gmRemain1 = 1;
                    if (gmRemain1 == 0) gmRemain1 = 2;      //=3byte MIDI Msgs
                    gmDataLen1 = gmRemain1;
                }
            }
            byteCount++;
        }
//distribute bytes from Merge Master: (no filter in legacy mode)
        if ((recvword = pullMIDI5RxBuf())) {
            recvbyte = (uint8_t)(recvword & 0x00FF);
            sendB1(recvbyte);
            sendB3(recvbyte);
            if (gOpMode == 3) enCodeMidi(recvbyte);
            else sendEtherByte(recvbyte);
        }
        return;
    }

    if (gmSchedulerStage == 1) {
       byteCount = 0;
        while (byteCount < 3)  {    //process max.3bytes per task robin round

            if ((recvword = pullMIDI3RxBuf()) == 0) break;
            recvbyte = (uint8_t)(recvword & 0x00FF);

            if (recvbyte < 0x80) {
                if (gmRemain3 == 4)  {          //only possible with long SysEX
                    msgMidi3();                 //merge is not safe anymore !!
                    gmRemain3 = USART_BUFLEN;
                    gmDataLen3 = USART_BUFLEN;
                }
                if (gmRemain3 == 0)  break;                 //ignore wild bytes
                if (gmFillStatus3) {                        //is in RunningState
                    *gmMIDI3_MSGHEAD++ = gmStatus3 | gmChannel3;
                    gmFillStatus3 = 0;
                }
/*  ERROR Revnum 3!!
                if (gmFillStatus1) {
                    *gmMIDI1_MSGHEAD++ = gmStatus1 | gmChannel1;
                    gmFillStatus1 = 0;
                }
*/
                *gmMIDI3_MSGHEAD++ = recvbyte;
                gmRemain3--;
                if (gmRemain3 == 0)  {
                    legacyMsgMidi3();
                    gmRemain3 = gmDataLen3;                 //for Running State
                    gmFillStatus3 = 0xFF;
                    break;
                }
            }
            else {
                gmFillStatus3 = 0;              //start collecting new message
                if ((recvbyte == 0xF7) && (gmStatus3 == 0xF0)) { //end of Sysex
                    *gmMIDI3_MSGHEAD++ = recvbyte;
                    legacyMsgMidi3();
                    gmRemain3 = 0;
                    gmDataLen3 = 0;
                    break;
                }
                else if (recvbyte >= 0xF4) sendB5(recvbyte); //single byte msg
                            //MIDI5 is exclusive msg collector for all MIDI IN
                            //even forward undefined messages immediately
                else {
                    gmMIDI3_MSGHEAD = gmMIDI3_MSGBUF; //ignore uncomplete msgs
                    *gmMIDI3_MSGHEAD++ = recvbyte;
                    gmRemain3 = 0;
                    gmStatus3 = recvbyte & 0xF0;
                    gmChannel3 = recvbyte & 0x0F;
                    if (recvbyte == 0xF0) gmRemain3 = USART_BUFLEN; //max length
                    if ((recvbyte == 0xF1) || (recvbyte == 0xF3)) gmRemain3 = 1;
                    if ((gmStatus3 == 0xC0)||(gmStatus3 == 0xD0)) gmRemain3 = 1;
                    if (gmRemain3 == 0) gmRemain3 = 2;      //=3byte MIDI Msgs
                    gmDataLen3 = gmRemain3;
                }
            }
            byteCount++;
        }
//distribute bytes from Merge Master: (no filter in legacy mode)
//checkUartFree unnecessary in this case, because input is as slow as output
        if ((recvword = pullMIDI5RxBuf())) {
            recvbyte = (uint8_t)(recvword & 0x00FF);
            sendB1(recvbyte);
            sendB3(recvbyte);
            if (gOpMode == 3) enCodeMidi(recvbyte);
            else sendEtherByte(recvbyte);
        }
        return;
    }

    if (gmSchedulerStage == 2) {
        byteCount = 0;
        while (byteCount < 3)  {      //process max.3bytes per task robin round

            if (gOpMode == 3) {      //USB is 3rd "merged"
                if ((recvword = pullUsbRxBuf()) == 0) break;
            }
            else  {                   //(gOpMode==0) Ethernet is 5rd "merged"
            if ((recvword = pullSpiRxBuf()) == 0) break;
            }
            recvbyte = (uint8_t)(recvword & 0x00FF);
            if (recvbyte < 0x80) {
                if (gmRemainX == 4)  {          //only possible with long SysEX
                    msgMidiX();                 //merge is not safe anymore !!
                    gmRemainX = USART_BUFLEN;
                    gmDataLenX = USART_BUFLEN;
                }
                if (gmRemainX == 0)  break;                 //ignore wild bytes

                if (gmFillStatusX) {                        //is in RunningState
                    *gmMIDI1_MSGHEAD++ = gmStatusX | gmChannelX;
                    gmFillStatusX = 0;
                }
                *gmMIDIX_MSGHEAD++ = recvbyte;
                gmRemainX--;
                if (gmRemainX == 0)  {
                    legacyMsgMidiX();
                    gmRemainX = gmDataLenX;                 //for Running State
                    gmFillStatusX = 0xFF;
                    break;
                }
            }
            else {
                gmFillStatusX = 0;              //start collecting new message
                if ((recvbyte == 0xF7) && (gmStatusX == 0xF0)) { //end of Sysex
                    *gmMIDIX_MSGHEAD++ = recvbyte;
                    legacyMsgMidiX();
                    gmRemainX = 0;
                    gmDataLenX = 0;
                    break;
                }
                else if (recvbyte >= 0xF4) sendB5(recvbyte); //single byte msg
                            //MIDI5 is exclusive msg collector for all MIDI IN
                            //even forward undefined messages immediately
                else {
                    gmMIDIX_MSGHEAD = gmMIDIX_MSGBUF; //ignore uncomplete msgs
                    *gmMIDIX_MSGHEAD++ = recvbyte;
                    gmRemainX = 0;
                    gmStatusX = recvbyte & 0xF0;
                    gmChannelX = recvbyte & 0x0F;
                    if (recvbyte == 0xF0) gmRemainX = USART_BUFLEN; //max length
                    if ((recvbyte == 0xF1) || (recvbyte == 0xF3)) gmRemainX = 1;
                    if ((gmStatusX == 0xC0)||(gmStatusX == 0xD0)) gmRemainX = 1;
                    if (gmRemainX == 0) gmRemainX = 2;      //=3byte MIDI Msgs
                    gmDataLenX = gmRemainX;
                }
            }
            byteCount++;
        }
//distribute bytes from Merge Master: (no filter in legacy mode)
        if ((recvword = pullMIDI5RxBuf())) {
            recvbyte = (uint8_t)(recvword & 0x00FF);
            sendB1(recvbyte);
            sendB3(recvbyte);
            if (gOpMode == 3) enCodeMidi(recvbyte);
            else sendEtherByte(recvbyte);
        }
        return;
    }
}


void legacyMsgMidi1(void)  // ***********************************************
// **************************************************************************
{
uint16_t toSend, offset;

    toSend = (uint16_t)(gmMIDI1_MSGHEAD - gmMIDI1_MSGBUF);
    offset = 0;
    while (toSend)  {
        sendB5(gmMIDI1_MSGBUF[offset]);
        toSend--;
        offset++;
    }
    gmMIDI1_MSGHEAD = gmMIDI1_MSGBUF;           //reset Pointer
}


void legacyMsgMidi3(void)  // ***********************************************
// **************************************************************************
{
uint16_t toSend, offset;

    toSend = (uint16_t)(gmMIDI3_MSGHEAD - gmMIDI3_MSGBUF);
    offset = 0;
    while (toSend)  {
        sendB5(gmMIDI3_MSGBUF[offset]);
        toSend--;
        offset++;
    }
    gmMIDI3_MSGHEAD = gmMIDI3_MSGBUF;           //reset Pointer
}


void legacyMsgMidiX(void)  // ***********************************************
// **************************************************************************
{
uint16_t toSend, offset;

    toSend = (uint16_t)(gmMIDIX_MSGHEAD - gmMIDIX_MSGBUF);
    offset = 0;
    while (toSend)  {
        sendB5(gmMIDIX_MSGBUF[offset]);
        toSend--;
        offset++;
    }
    gmMIDIX_MSGHEAD = gmMIDIX_MSGBUF;           //reset Pointer
}


// ##########################################################################
