// User Interface routines for "MIDI Merger"  *********************************
//
// The MIDI Merger
// merges up to 4 conventional MIDI Inputs into a common MIDI message stream
// to a PC via Ethernet (Multicast or standard UDP) or via USB MIDI Class.
// Vice versa a stream of MIDI messages from a PC is distributed to
// up to 4 conventional MIDI Outputs with channel filter per output.
//
// The design is based on the STM32 Nucleo-L476 module plus a specific PCB.
//
// Author and (c)2017-18 Wolfgang Schemmert. The copyright claim and
// the GPL-3 Licence is restricted to this file and the depending files
// "main.c", "main.h", "usb-otg.c", "midi.c" and "ether.c".
// Other depending material is published and copyrighted
// by STMicroelectronics, Atollic AB and EMBITZ.org
//
// Contact: <www.midi-and-more.de>
// A detailled construction and operation manual is provided at this website
//
// Note: at present state of art, only the DEBUG object code is working.
// The RELEASE version compiled with EmBitz 1.11 does not work. Reason unkown.
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32L476xx.h"
#include "globaldefines.h"

// ************************************

extern struct PARAM{
uint32_t Vid;
uint32_t Pid;
uint32_t MacHi;
uint32_t MacLo;
uint32_t Gateway;
uint32_t LocalIP;
uint32_t Subnet;
uint32_t UdpDestIP;
uint32_t UdpDestPort;
uint32_t UdpLocalPort;
uint32_t McastIP;
uint32_t McastDestPort;
uint32_t McastLocalPort;
uint32_t LanMode;           //includes BlockF68E, MIDI4,5 Split configuration
uint32_t Midi13Mask;
uint32_t Midi45Mask;
uint32_t EthUsbMask;
uint32_t TcpPort;
uint32_t DisconTime;
uint32_t ComBaud;
uint32_t MidiBaud;
uint32_t Padding;
} gComParam, gComParamShadow;

extern void popFlashPage24(void);
extern uint8_t pushFlashPage24(void);
extern void servSystem(void);
extern void msDelay(uint32_t);
extern void wait1ms(void);
extern void wait6us(void);

//RS-232
extern void initUsart2(void);
extern void sendSetupB(uint8_t);
extern void sendN8(uint8_t);
extern void sendX16(uint16_t);
extern void sendX32(uint32_t);
extern void sendN32(uint32_t);
extern void sendS32(uint32_t);

extern void sendString(const uint8_t *);
extern uint16_t pullSetupRxBuf(void);

//USB
extern uint8_t guDeviceClass;

//midi.c
uint8_t gmSplitMidi4, gmSplitMidi5;
extern uint8_t gmBlockF6, gmBlockF8, gmBlockFE;
uint16_t gmMask1, gmMask3, gmMask4, gmMask5, gmMaskEth, gmMaskUsb;

// ************************************

uint32_t gcEtherIP;

const uint8_t gcSetStr[] = {"set"};
const uint8_t gcVidPidStr[] = {" USB Vid/Pid of this device (hex)"};
const uint8_t gcMacIdStr[] = {" Ethernet MAC ID (hex)m5.m4.m3.m2.m1.m0)"};
const uint8_t gcGatewayStr[] = {" Gateway IP (ip3.ip2.ip1.ip0) "};
const uint8_t gcLocalIpStr[] = {" IP of this device (ip3.ip2.ip1.ip0)"};
const uint8_t gcSubnetStr[] = {" Subnet Mask of this device (m3.m2.m1.m0)"};
const uint8_t gcMcastIPStr[] = {" Multicast UDP MIDI Merge IP(ip3.ip2.ip1.ip0)"};
const uint8_t gcMcastDestPortStr[] = {" MIDI(in)->Multicast Destination(Tx) Port number (Merge)"};
const uint8_t gcMcastLocalPortStr[] = {" Multicast Local(Rx)->MIDI(out) Port number (Distribution/Split)"};
const uint8_t gcUdpDestIpStr[] = {" Std.UDP/Com Destination IP incl.Broadcast(ip3.ip2.ip1.ip0)"};
const uint8_t gcUdpLocalPortStr[] = {" UDP Local(Rx) Port number (std.UDP, not Multicast)"};
const uint8_t gcUdpDestPortStr[] = {" UDP Destination(Tx) Port number"};
const uint8_t gcLanModeStr[] = {" Lan Mode: 0=UDP Multicast Merge, 1=std.UDP Merge, 2=TCP&UDP COM"};
const uint8_t gcSplitMidi4Str[] = {" Split/Distribute MIDI4 IN to MIDI1,3,5 OUT: 0=NO, 1=YES"};
const uint8_t gcSplitMidi5Str[] = {" Split/Distribute MIDI5 IN to MIDI1,3,4 OUT: 0=NO, 1=YES"};
const uint8_t gcBlockF68EStr[] = {" Bit0:block TimingClk, Bit1:block ActSens, Bit2:block TuneReq"};
const uint8_t gcMidi1MaskStr[] = {" MIDI1 OUT Filter for CH.1-16 (4 hex nibbles XXXX)"};
const uint8_t gcMidi3MaskStr[] = {" MIDI3 OUT Filter for CH.1-16 (4 hex nibbles XXXX)"};
const uint8_t gcMidi4MaskStr[] = {" MIDI4 OUT Filter for CH.1-16 (4 hex nibbles XXXX)"};
const uint8_t gcMidi5MaskStr[] = {" MIDI5 OUT Filter for CH.1-16 (4 hex nibbles XXXX)"};
const uint8_t gcEtherMaskStr[] = {" Ethernet OUT Filter for CH.1-16 (4 hex nibbles XXXX)"};
const uint8_t gcUsbMaskStr[] = {" USB OUT Filter for CH.1-16 (4 hex nibbles XXXX)"};
const uint8_t gcTcpPortStr[] = {" TCP/Com Port number (Telnet/COM port only, not for MIDI) !!"};
const uint8_t gcDisconStr[] = {" TCP Disconnect Time (0 ... 65535 seconds, 0=no auto disconnect)"};
const uint8_t gcComBaudStr[] = {" COM4 Baudrate (UDP&TCP COM + Setup only): first 2 digits,MIDI=31"};
const uint8_t gcMidiBaudStr[] = {" COM4 Baudrate (MIDI Merger operation): first 2 digits,MIDI=31"};

uint8_t  CMD_Interpreter(uint8_t);
uint16_t gcHexWord;

uint32_t parseHexByte(void);
uint8_t parseHexWord(void);
uint32_t parseHexNibble(void);
uint32_t parseDeciNum(uint32_t);
void sendS8(uint32_t);
void sendS16(uint32_t);
uint8_t setVidPid(void);
uint8_t setMacID(void);
uint8_t setEtherIP(void);
void setLanMode(void);
void setSplitMidi4(void);
void setSplitMidi5(void);
void setBlockF68E(void);
void setComBaudrate(void);
void setMidiBaudrate(void);
void listCommands(void);
void listSetup(void);


uint8_t  CMD_Interpreter(uint8_t opcode)   // ********************************
// ***************************************************************************
{
uint32_t  parameter;
uint16_t recvword;
const uint8_t SaveAgainStr[] = {"Save to Flash?->press again "};
const uint8_t LoadAgainStr[] = {"Reload Setup from Flash?->press again "};
const uint8_t NotSavedStr[] = {" NOT SAVED! "};
const uint8_t NotLoadedStr[] = {" NOT LOADED! "};
const uint8_t ErrorStr[] = {" Flash Write Error, do Setup again! "};

    if ((opcode > 0x60) && (opcode <0x7B)) opcode -= 0x20;
    if ((opcode == 0x0D) || (opcode == 0x0A)) return 1;

    switch(opcode)  {

        case '!':                   //list actual configuration
            listSetup();
			sendSetupB(0x0D);
            return 1;

        case 'A':
            sendString(gcSetStr);
            sendString(gcMacIdStr);
			sendSetupB(':');
            sendSetupB(0x20);
            setMacID();
            sendSetupB(0x0D);
            return 1;

        case 'G':
            sendString(gcSetStr);
            sendString(gcGatewayStr);
            sendSetupB(':');
			sendSetupB(0x20);
            setEtherIP();
            gComParamShadow.Gateway = gcEtherIP;
			sendSetupB(0x0D);
            return 1;

       case 'S':
            sendString(gcSetStr);
            sendString(gcSubnetStr);
            sendSetupB(':');
			sendSetupB(0x20);
            setEtherIP();
            gComParamShadow.Subnet = gcEtherIP;
			sendSetupB(0x0D);
            return 1;

        case 'I':
            sendString(gcSetStr);
            sendString(gcLocalIpStr);
            sendSetupB(':');
			sendSetupB(0x20);
            setEtherIP();
            gComParamShadow.LocalIP = gcEtherIP;
			sendSetupB(0x0D);
            return 1;

        case 'M':
            sendString(gcSetStr);
            sendString(gcMcastIPStr);
            sendSetupB(':');
			sendSetupB(0x20);
            setEtherIP();
            gComParamShadow.McastIP = gcEtherIP;
			sendSetupB(0x0D);
            return 1;

        case 'H':
            sendString(gcSetStr);
           	sendString(gcMcastLocalPortStr);
 			sendSetupB(':');
 			sendSetupB(0x20);
            if ((parameter = parseDeciNum(65535)) < 65536)
                                    gComParamShadow.McastLocalPort = parameter;
 			 sendSetupB(0x0D);
             return 1;

       case 'D':
            sendString(gcSetStr);
           	sendString(gcMcastDestPortStr);
 			sendSetupB(':');
 			sendSetupB(0x20);
            if ((parameter = parseDeciNum(65535)) < 65536)
                                    gComParamShadow.McastDestPort = parameter;
 			 sendSetupB(0x0D);
             return 1;

        case 'U':
            sendString(gcSetStr);
            sendString(gcUdpDestIpStr);
            sendSetupB(':');
			sendSetupB(0x20);
            setEtherIP();
            gComParamShadow.UdpDestIP = gcEtherIP;
			sendSetupB(0x0D);
            return 1;

        case 'T':
            sendString(gcSetStr);
           	sendString(gcUdpDestPortStr);
 			sendSetupB(':');
 			sendSetupB(0x20);
            if ((parameter = parseDeciNum(65535)) < 65536)
                                    gComParamShadow.UdpDestPort = parameter;
 			 sendSetupB(0x0D);
             return 1;

        case 'R':
            sendString(gcSetStr);
         	sendString(gcUdpLocalPortStr);
			sendSetupB(':');
			sendSetupB(0x20);
            if ((parameter = parseDeciNum(65535)) < 65536)
                                    gComParamShadow.UdpLocalPort = parameter;
			sendSetupB(0x0D);
            return 1;

       case 'F':
            sendString(gcSetStr);
         	sendString(gcBlockF68EStr);
			sendSetupB(':');
			sendSetupB(0x20);
            setBlockF68E();
			sendSetupB(0x0D);
            return 1;

      case 'J':
            sendString(gcSetStr);
         	sendString(gcSplitMidi4Str);
			sendSetupB(':');
			sendSetupB(0x20);
            setSplitMidi4();
			sendSetupB(0x0D);
            return 1;

        case 'K':
            sendString(gcSetStr);
         	sendString(gcSplitMidi5Str);
			sendSetupB(':');
			sendSetupB(0x20);
            setSplitMidi5();
			sendSetupB(0x0D);
            return 1;

        case 'L':
            sendString(gcSetStr);
            sendString(gcLanModeStr);
            sendSetupB(':');
			sendSetupB(0x20);
            setLanMode();
			sendSetupB(0x0D);
            return 1;


        case 'P':
            sendString(gcSetStr);
           	sendString(gcTcpPortStr);
 			sendSetupB(':');
 			sendSetupB(0x20);
            if ((parameter = parseDeciNum(65535)) < 65536)
                                    gComParamShadow.TcpPort = parameter;
 			 sendSetupB(0x0D);
             return 1;

       case 'X':
            sendString(gcSetStr);
           	sendString(gcDisconStr);
 			sendSetupB(':');
 			sendSetupB(0x20);
            if ((parameter = parseDeciNum(65535)) < 65536)
                                    gComParamShadow.DisconTime = parameter;
 			 sendSetupB(0x0D);
             return 1;

       case 'B':
            sendString(gcSetStr);
            sendString(gcComBaudStr);
			sendSetupB(':');
            sendSetupB(0x20);
            setComBaudrate();
            sendSetupB(0x0D);
            return 1;

       case 'N':
            sendString(gcSetStr);
            sendString(gcMidiBaudStr);
			sendSetupB(':');
            sendSetupB(0x20);
            setMidiBaudrate();
            sendSetupB(0x0D);
            return 1;

        case 'V':
            sendString(gcSetStr);
         	sendString(gcVidPidStr);
			sendSetupB(':');
			sendSetupB(0x20);
            if (setVidPid() == 0) sendSetupB('?');
			sendSetupB(0x0D);
            return 1;

        case '1':
            sendString(gcSetStr);
         	sendString(gcMidi1MaskStr);
			sendSetupB(':');
			sendSetupB(0x20);
            if (parseHexWord()) {
                gComParamShadow.Midi13Mask &= 0x0000FFFF;
                gComParamShadow.Midi13Mask |= gcHexWord<<16;
                gmMask1 = gcHexWord;
            }
            else sendSetupB('?');
			sendSetupB(0x0D);
            return 1;

          case '3':
            sendString(gcSetStr);
         	sendString(gcMidi3MaskStr);
			sendSetupB(':');
			sendSetupB(0x20);
             if (parseHexWord()) {
                gComParamShadow.Midi13Mask &= 0xFFFF0000;
                gComParamShadow.Midi13Mask |= gcHexWord;
                gmMask3 = gcHexWord;
            }
            else sendSetupB('?');
			sendSetupB(0x0D);
            return 1;

         case '4':
            sendString(gcSetStr);
         	sendString(gcMidi4MaskStr);
			sendSetupB(':');
			sendSetupB(0x20);
            if (parseHexWord()) {
                gComParamShadow.Midi45Mask &= 0x0000FFFF;
                gComParamShadow.Midi45Mask |= gcHexWord<<16;
                gmMask4 = gcHexWord;
            }
            else sendSetupB('?');
			sendSetupB(0x0D);
            return 1;

         case '5':
            sendString(gcSetStr);
         	sendString(gcMidi5MaskStr);
			sendSetupB(':');
			sendSetupB(0x20);
              if (parseHexWord()) {
                gComParamShadow.Midi45Mask &= 0xFFFF0000;
                gComParamShadow.Midi45Mask |= gcHexWord;
                gmMask5 = gcHexWord;
            }
            else sendSetupB('?');
			sendSetupB(0x0D);
            return 1;

         case 'Y':
            sendString(gcSetStr);
         	sendString(gcEtherMaskStr);
			sendSetupB(':');
			sendSetupB(0x20);
             if (parseHexWord()) {
                gComParamShadow.EthUsbMask &= 0x0000FFFF;
                gComParamShadow.EthUsbMask |= gcHexWord<<16;
                gmMaskEth = gcHexWord;
            }
            else sendSetupB('?');
			sendSetupB(0x0D);
            return 1;

         case 'Z':
            sendString(gcSetStr);
         	sendString(gcUsbMaskStr);
			sendSetupB(':');
			sendSetupB(0x20);
              if (parseHexWord()) {
                gComParamShadow.EthUsbMask &= 0xFFFF0000;
                gComParamShadow.EthUsbMask |= gcHexWord;
                gmMaskUsb = gcHexWord;
            }
            else sendSetupB('?');
			sendSetupB(0x0D);
            return 1;

        case '?':
            sendSetupB(0x0D);
            listCommands();
            sendSetupB(0x0D);
            return 1;

        case '@':
            msDelay(300);
            if (pullSetupRxBuf() != 0) {
                sendSetupB('?');        //pause <300ms too short
                sendSetupB(0x20);
            }
            sendString(LoadAgainStr);
            do  {
                recvword = pullSetupRxBuf();
            }  while (recvword == 0) ;
            if (recvword == 0x8040)  {              //'@'
                popFlashPage24();
                sendSetupB(0x20);
                sendSetupB('o');
                sendSetupB('k');
            }
            else  sendString(NotLoadedStr);
            return 1;

        case '|':
 //           sendSetupB(0x0D);
            msDelay(300);
            if (pullSetupRxBuf() != 0) {
            sendSetupB('?');        //pause <300ms too short
            sendSetupB(0x20);
            }
            sendString(SaveAgainStr);
            do  {
                recvword = pullSetupRxBuf();
            }  while (recvword == 0) ;
            if (recvword == 0x807C)  {      //'|'
                if (pushFlashPage24())  {
                    sendSetupB(0x20);
                    sendSetupB('o');
                    sendSetupB('k');
                    sendSetupB(0x0D);
                }
                else sendString(ErrorStr);
            }
            else  sendString(NotSavedStr);
            return 1;

        default: return  0;
    }
    return  0;                                 //formally, is never reached
}


uint32_t parseHexNibble(void)   //*******************************************
// **************************************************************************
{
uint16_t recvword;
uint32_t nibble;

    while ((recvword = pullSetupRxBuf()) == 0) ;
    nibble = (uint32_t)(recvword & 0x00FF);
    if ((nibble == 0x0A)||(nibble == 0x0D)||(nibble == '.')) return 0xFFFFFFFE;
    if  ((nibble >= 0x30) && (nibble <= 0x39))  {
        sendSetupB((uint8_t)nibble);
        return (nibble - 0x30);
    }
    if ((nibble > 0x60) && (nibble < 0x7B))  nibble -= 0x20; //force uppercase
    if ((nibble >= 'A') && (nibble <= 'F')) {
        sendSetupB((uint8_t)nibble);
        return (nibble - 0x37);
    }
    else return 0xFFFFFFFF;

}


uint32_t  parseHexByte(void)   // ********************************************
// ***************************************************************************
{
uint32_t nibble1, nibble2;

        if ((nibble1 = parseHexNibble())  == 0xFFFFFFFE) return 0xFFFFFFFF;
        else if (nibble1 > 0x0000000F) return 0xFFFFFFFF;
        if ((nibble2 = parseHexNibble()) == 0xFFFFFFFE) return nibble1;
        else if (nibble2 > 0x0000000F)  return 0xFFFFFFFF;
        if (parseHexNibble() != 0xFFFFFFFE) return 0xFFFFFFFF;
        return (uint32_t)((nibble1 << 4) | nibble2);
}


uint8_t parseHexWord(void)  // **********************************************
// **************************************************************************
{
uint32_t nibble;

    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    gcHexWord = (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    gcHexWord = (gcHexWord << 4) + (uint16_t)nibble;                // *16
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    gcHexWord = (gcHexWord << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    gcHexWord = (gcHexWord << 4) + (uint16_t)nibble;
    sendSetupB(' ');
    return 1;
}

uint32_t parseDeciNum(uint32_t limit)   // **********************************
//unsigned, 0 ...max 99.999
// **************************************************************************
{
uint32_t decinum;
uint16_t recvword;
uint8_t digit;

    if (limit >= 99000) return 0xFFFFFFFF;
    decinum = 0;
    while (decinum <= limit)  {
        while ((recvword = pullSetupRxBuf()) == 0) ;
        digit = (uint8_t)(recvword & 0x00FF);
        if ((digit == 0x0A) || (digit==0x0D) || (digit== '.')) {
            return decinum;
        }
        if ((digit < 0x30) || (digit > 0x39 )) {
            sendSetupB('?');
            return 0xFFFFFFFF;
        }
        sendSetupB(digit);
        digit -= 0x30;     //ASCII>num
        decinum  = (decinum * 10) + digit;
        if (decinum > limit) {
            sendSetupB('?');
            return 0xFFFFFFFF;
        }
    }
    return decinum;
}


void sendS8(uint32_t number)   // ************************************************
// **************************************************************************
{
    if (number < 100) sendSetupB('0');
    if (number < 10) sendSetupB('0');
    sendN32(number);
}


void sendS16(uint32_t number)   // ************************************************
// **************************************************************************
{
    sendS32(number);
    if (number < 10000) sendSetupB(0x20);
    if (number < 1000) sendSetupB(0x20);
    if (number < 100) sendSetupB(0x20);
    if (number < 10) sendSetupB(0x20);
}

uint8_t setVidPid(void)  // ****************************************************
// **************************************************************************
{
uint8_t nibble;
uint16_t Vid, Pid;
const uint8_t VidStr[] = {"Vid="};
const uint8_t PidStr[] = {"Pid="};
    sendString(VidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;                // *16
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;
    sendSetupB(' ');

    sendString(PidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;

    gComParamShadow.Vid = Vid;
    gComParamShadow.Pid = Pid;
    return 1;
}


uint8_t setMacID(void)  // *****************************************************
// **************************************************************************
{
uint32_t  macHi, macLo, temp;

    temp = parseHexByte();
    if (temp > 255) return 0;
    macHi = temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macHi = (macHi << 8) | temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;
// replace not before MAC ID completely parsed
    gComParamShadow.MacHi = macHi;
    gComParamShadow.MacLo = macLo;
    return 1;
}


uint8_t setEtherIP(void)  // ************************************************
// **************************************************************************
{
uint32_t  etherIP, temp;

    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = temp;
    sendSetupB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;
    sendSetupB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;
    sendSetupB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;
// replace not before ID completely parsed
    gcEtherIP = etherIP;
    return 1;
}


void setLanMode(void)  // ************************************************
// **************************************************************************
{
uint32_t mode;
    if ((mode = parseDeciNum(2)) > 2) {
        sendSetupB('?');
        return;
    }
    gComParamShadow.LanMode &=  0xFFFFFFF0;
    gComParamShadow.LanMode |=  mode;
}


void setSplitMidi4(void)  // ************************************************
// **************************************************************************
{
uint32_t mode;
    if ((mode = parseDeciNum(1)) > 1) {
        sendSetupB('?');
        return;
    }
    gmSplitMidi4 = (uint8_t)(mode & 1);
    gComParamShadow.LanMode &= 0xFFFFFF0F;
    gComParamShadow.LanMode |= mode << 4;

}


void setSplitMidi5(void)  // ************************************************
// **************************************************************************
{
uint32_t mode;
    if ((mode = parseDeciNum(1)) > 1) {
        sendSetupB('?');
        return;
    }
    gmSplitMidi5 = (uint8_t)(mode & 1);
    gComParamShadow.LanMode &= 0xFFFFF0FF;
    gComParamShadow.LanMode |= mode << 8;
}

void setBlockF68E(void)  // *************************************************
// **************************************************************************
{
uint32_t mode;
    if ((mode = parseDeciNum(7)) > 7) {
        sendSetupB('?');
        return;
    }
    gmBlockF6 = (uint8_t)(mode & 4);
    gmBlockFE = (uint8_t)(mode & 2);
    gmBlockF8 = (uint8_t)(mode & 1);
    gComParamShadow.LanMode &= 0xFFFF0FFF;
    gComParamShadow.LanMode |= mode << 12;
}

void setComBaudrate(void)  // **********************************************
// **************************************************************************
{
uint32_t baudrate;

    if ((baudrate = parseDeciNum(96)) > 96) {
        sendSetupB('?');
        return;
    }
    switch (baudrate) {
        case 11:
            gComParamShadow.ComBaud = baudrate;                 //115200 Baud
            break;
        case 19:
            gComParamShadow.ComBaud = baudrate;                 //19200 Baud
            break;
        case 23:
            gComParamShadow.ComBaud = baudrate;                 //230400 Baud
            break;
        case 31:
            gComParamShadow.ComBaud = baudrate;                 //31250 = MIDI
            break;
        case 38:
            gComParamShadow.ComBaud = baudrate;                 //38400 Baud
            break;
        case 57:
            gComParamShadow.ComBaud = baudrate;                 //57600 Baud
            break;
        case 96:
            gComParamShadow.ComBaud = baudrate;                 //9600 Baud
            break;
        default:
            sendSetupB('?');
            break;
    }
}


void setMidiBaudrate(void)  // **********************************************
// **************************************************************************
{
uint32_t baudrate;

    if ((baudrate = parseDeciNum(96)) > 96) {
        sendSetupB('?');
        return;
    }
    switch (baudrate) {
        case 11:
            gComParamShadow.MidiBaud = baudrate;                 //115200 Baud
            break;
        case 19:
            gComParamShadow.MidiBaud = baudrate;                 //19200 Baud
            break;
        case 23:
            gComParamShadow.MidiBaud = baudrate;                 //230400 Baud
            break;
        case 31:
            gComParamShadow.MidiBaud = baudrate;                 //31250 = MIDI
            break;
        case 38:
            gComParamShadow.MidiBaud = baudrate;                 //38400 Baud
            break;
        case 57:
            gComParamShadow.MidiBaud = baudrate;                 //57600 Baud
            break;
        case 96:
            gComParamShadow.MidiBaud = baudrate;                 //9600 Baud
            break;
        default:
            sendSetupB('?');
            break;
    }
}


void listCommands(void)  // *************************************************
// **************************************************************************
{
const uint8_t SetupStr[] = {"List of Setup Commands:"};
const uint8_t ActualStr[] = {"show Actual Setup"};
const uint8_t SaveFlashStr[] = {"save new setup permanently (will become active at next Power Up)"};
const uint8_t LoadFlashStr[] = {"reload setup from Flash (changes since last Power Up are cancelled)"};
const uint8_t TypeStr[] = "Midi Merger v" ;
const uint8_t CopyStr[] = " (c)2017 W.Schemmert";

    sendSetupB(0x0D);
    sendString(TypeStr);
    sendSetupB(HARDWARE_VERSION + 0x30);
    sendSetupB('.');
    sendSetupB(REVNUM + 0x30);
    sendString(CopyStr);

    sendSetupB(0x0D);
    sendString(SetupStr);
    sendSetupB(0x0D);

    sendSetupB('!');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(ActualStr);
    sendSetupB(0x0D);

    sendSetupB('V');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcVidPidStr);
    sendSetupB(0x0D);

    sendSetupB('A');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMacIdStr);
    sendSetupB(0x0D);

    sendSetupB('G');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcGatewayStr);
    sendSetupB(0x0D);

    sendSetupB('I');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcLocalIpStr);
    sendSetupB(0x0D);

    sendSetupB('S');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcSubnetStr);
    sendSetupB(0x0D);

    sendSetupB('M');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMcastIPStr);
    sendSetupB(0x0D);

    sendSetupB('H');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMcastLocalPortStr);
    sendSetupB(0x0D);

    sendSetupB('D');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMcastDestPortStr);
    sendSetupB(0x0D);

    sendSetupB('U');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcUdpDestIpStr);
    sendSetupB(0x0D);

    sendSetupB('R');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcUdpLocalPortStr);
    sendSetupB(0x0D);

    sendSetupB('T');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcUdpDestPortStr);
    sendSetupB(0x0D);

    sendSetupB('L');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcLanModeStr);
    sendSetupB(0x0D);

    sendSetupB('P');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcTcpPortStr);
    sendSetupB(0x0D);

    sendSetupB('X');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcDisconStr);
    sendSetupB(0x0D);

    sendSetupB('B');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcComBaudStr);
    sendSetupB(0x0D);

    sendSetupB('N');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMidiBaudStr);
    sendSetupB(0x0D);

    sendSetupB('J');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcSplitMidi4Str);
    sendSetupB(0x0D);

    sendSetupB('K');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcSplitMidi5Str);
    sendSetupB(0x0D);

    sendSetupB('F');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcBlockF68EStr);
    sendSetupB(0x0D);

    sendSetupB('1');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMidi1MaskStr);
    sendSetupB(0x0D);

    sendSetupB('3');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMidi3MaskStr);
    sendSetupB(0x0D);

    sendSetupB('4');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMidi4MaskStr);
    sendSetupB(0x0D);

    sendSetupB('5');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcMidi5MaskStr);
    sendSetupB(0x0D);

    sendSetupB('Y');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcEtherMaskStr);
    sendSetupB(0x0D);

    sendSetupB('Z');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(gcSetStr);
    sendString(gcUsbMaskStr);
    sendSetupB(0x0D);

    sendSetupB('@');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(LoadFlashStr);
    sendSetupB(0x0D);

    sendSetupB('|');
    sendSetupB(':');
    sendSetupB(0x20);
    sendString(SaveFlashStr);
//    sendSetupB(0x0D);
}


void listSetup(void)  // *****************************************************
// **************************************************************************
{
const uint8_t SetupStr[] = {"List of Actual Setup:"};

    sendSetupB(0x0D);
    sendString(SetupStr);
    sendSetupB(0x0D);

    sendSetupB('V');
    sendSetupB(':');
    sendSetupB(0x20);
    sendX16(gComParamShadow.Vid & 0x0000FFFF);
    sendSetupB('/');
    sendSetupB(0x20);
    sendX16(gComParamShadow.Pid & 0x0000FFFF);
    sendString(gcVidPidStr);
    sendSetupB(0x0D);

    sendSetupB('A');
    sendSetupB(':');
    sendSetupB(0x20);
    sendN8((gComParamShadow.MacHi>>8) & 0xFF);
    sendSetupB('.');
    sendN8(gComParamShadow.MacHi & 0xFF);
    sendSetupB('.');
    sendN8((gComParamShadow.MacLo >> 24) & 0xFF);
    sendSetupB('.');
    sendN8((gComParamShadow.MacLo >> 16) & 0xFF);
    sendSetupB('.');
    sendN8((gComParamShadow.MacLo >> 8) & 0xFF);
    sendSetupB('.');
    sendN8(gComParamShadow.MacLo & 0xFF);
    sendString(gcMacIdStr);
    sendSetupB(0x0D);

    sendSetupB('G');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.Gateway >> 24) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.Gateway >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.Gateway >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.Gateway & 0xFF);
    sendString(gcGatewayStr);
    sendSetupB(0x0D);

    sendSetupB('I');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.LocalIP >> 24) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.LocalIP >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.LocalIP >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.LocalIP & 0xFF);
    sendString(gcLocalIpStr);
    sendSetupB(0x0D);

    sendSetupB('S');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.Subnet >> 24) &0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.Subnet >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.Subnet >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.Subnet & 0xFF);
    sendString(gcSubnetStr);
    sendSetupB(0x0D);

    sendSetupB('M');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.McastIP >> 24) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.McastIP >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.McastIP >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.McastIP & 0xFF);
    sendString(gcMcastIPStr);
    sendSetupB(0x0D);

    sendSetupB('H');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.McastLocalPort & 0x0000FFFF);
    sendString(gcMcastLocalPortStr);
    sendSetupB(0x0D);

    sendSetupB('D');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.McastDestPort & 0x0000FFFF);
    sendString(gcMcastDestPortStr);
    sendSetupB(0x0D);

    sendSetupB('U');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.UdpDestIP >> 24) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.UdpDestIP >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.UdpDestIP >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.UdpDestIP & 0xFF);
    sendString(gcUdpDestIpStr);
    sendSetupB(0x0D);

    sendSetupB('R');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.UdpLocalPort & 0x0000FFFF);
    sendString(gcUdpLocalPortStr);
    sendSetupB(0x0D);

    sendSetupB('T');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.UdpDestPort & 0x0000FFFF);
    sendString(gcUdpDestPortStr);
    sendSetupB(0x0D);

    sendSetupB('L');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.LanMode & 0x0000000F);
    sendString(gcLanModeStr);
    sendSetupB(0x0D);

    sendSetupB('P');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.TcpPort & 0x0000FFFF);
    sendString(gcTcpPortStr);
    sendSetupB(0x0D);

    sendSetupB('X');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.DisconTime & 0x0000FFFF);
    sendString(gcDisconStr);
    sendSetupB(0x0D);

    sendSetupB('B');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupB((gComParamShadow.ComBaud/10)  + 0x30);
    sendSetupB((gComParamShadow.ComBaud %10) + 0x30);
    switch (gComParamShadow.ComBaud) {
        case 11:
            sendSetupB('5');
            sendSetupB('2');
            sendSetupB('0');
            sendSetupB('0');
            break;
        case 19:
            sendSetupB('2');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 23:
            sendSetupB('0');
            sendSetupB('4');
            sendSetupB('0');
            sendSetupB('0');
            break;
        case 31:
            sendSetupB('M');
            sendSetupB('I');
            sendSetupB('D');
            sendSetupB('I');
            break;
        case 38:
            sendSetupB('4');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 57:
            sendSetupB('6');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 96:
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            sendSetupB(0x20);
            break;
        default:
            sendSetupB('?');
            sendSetupB('?');
            sendSetupB('?');
            sendSetupB(0x20);
    }
    sendString(gcComBaudStr);
    sendSetupB(0x0D);

    sendSetupB('N');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupB((gComParamShadow.MidiBaud/10)  + 0x30);
    sendSetupB((gComParamShadow.MidiBaud %10) + 0x30);
    switch (gComParamShadow.MidiBaud) {
        case 11:
            sendSetupB('5');
            sendSetupB('2');
            sendSetupB('0');
            sendSetupB('0');
            break;
        case 19:
            sendSetupB('2');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 23:
            sendSetupB('0');
            sendSetupB('4');
            sendSetupB('0');
            sendSetupB('0');
            break;
        case 31:
            sendSetupB('M');
            sendSetupB('I');
            sendSetupB('D');
            sendSetupB('I');
            break;
        case 38:
            sendSetupB('4');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 57:
            sendSetupB('6');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 96:
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            sendSetupB(0x20);
            break;
        default:
            sendSetupB('?');
            sendSetupB('?');
            sendSetupB('?');
            sendSetupB(0x20);
    }
    sendString(gcMidiBaudStr);
    sendSetupB(0x0D);

    sendSetupB('J');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16((gComParamShadow.LanMode & 0x000000F0)>>4);
    sendString(gcSplitMidi4Str);
    sendSetupB(0x0D);

    sendSetupB('K');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16((gComParamShadow.LanMode & 0x00000F00)>>8);
    sendString(gcSplitMidi5Str);
    sendSetupB(0x0D);

    sendSetupB('F');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16((gComParamShadow.LanMode & 0x0000F000)>>12);
    sendString(gcBlockF68EStr);
    sendSetupB(0x0D);

    sendSetupB('1');
    sendSetupB(':');
    sendSetupB(0x20);
    sendX16(gComParamShadow.Midi13Mask >>16) ;
    sendSetupB(0x20);
    sendString(gcMidi1MaskStr);
    sendSetupB(0x0D);

    sendSetupB('3');
    sendSetupB(':');
    sendSetupB(0x20);
    sendX16(gComParamShadow.Midi13Mask & 0x0000FFFF) ;
    sendSetupB(0x20);
    sendString(gcMidi3MaskStr);
    sendSetupB(0x0D);

    sendSetupB('4');
    sendSetupB(':');
    sendSetupB(0x20);
    sendX16(gComParamShadow.Midi45Mask >>16) ;
    sendSetupB(0x20);
    sendString(gcMidi4MaskStr);
    sendSetupB(0x0D);

    sendSetupB('5');
    sendSetupB(':');
    sendSetupB(0x20);
    sendX16(gComParamShadow.Midi45Mask & 0x0000FFFF) ;
    sendSetupB(0x20);
    sendString(gcMidi5MaskStr);
    sendSetupB(0x0D);

    sendSetupB('Y');
    sendSetupB(':');
    sendSetupB(0x20);
    sendX16(gComParamShadow.EthUsbMask >>16) ;
    sendSetupB(0x20);
    sendString(gcEtherMaskStr);
    sendSetupB(0x0D);

    sendSetupB('Z');
    sendSetupB(':');
    sendSetupB(0x20);
    sendX16(gComParamShadow.EthUsbMask & 0x0000FFFF) ;
    sendSetupB(0x20);
    sendString(gcUsbMaskStr);
//    sendSetupB(0x0D);

}

// ##########################################################################
